
#define OBJ_STRUCT_LABEL_LEN 80
typedef struct {
  char label[OBJ_STRUCT_LABEL_LEN+1];
  COLOUR col;
  float alpha;
  int nverts;
  XYZ *verts;
  int nfacets;
  int *facets; // 3 * int per facet
  XYZ minP, maxP; // min,max coordinates
  XYZ meanP; // mean (~central) coordinate
  XYZ *norms;
} OBJ_STRUCT;

typedef struct {
  int nverts;
  int *vert_indices;
  float *vert_weights;
  float minW, maxW;
} OBJ_WGT_STRUCT;

OBJ_STRUCT *loadObj(char *filename, char *label, float r, float g, float b, float a);
OBJ_STRUCT *loadObjFromFS(char *filename, char *label, float r, float g, float b, float a);
OBJ_WGT_STRUCT *loadObjWgtFromFS(char *filename);
OBJ_STRUCT *loadObjFromSTL(char *filename, char *label, float r, float g, float b, float a);
void calcObjMinMaxMean(OBJ_STRUCT *obj);
void translateObj(OBJ_STRUCT *obj, XYZ vec);
void drawObj(OBJ_STRUCT *obj); //, COLOUR c);
void drawObjWgts(OBJ_STRUCT *obj, OBJ_WGT_STRUCT *wgt, int wgted_vx_only,
		 int solid_col_idx);


OBJ_STRUCT *loadObj(char *filename, char *label, float r, float g, float b, float a) {
  
  OBJ_STRUCT *obj = (OBJ_STRUCT *)malloc(1 * sizeof(OBJ_STRUCT));

  strncpy(obj->label, label, OBJ_STRUCT_LABEL_LEN);
  obj->col.r = r;
  obj->col.g = g;
  obj->col.b = b;
  obj->alpha = a;

  obj->nverts = 0;
  obj->verts = NULL;
  obj->nfacets = 0;
  obj->facets = NULL;

  obj->norms = NULL;

  int lcount = 0;
  char line[401];
  char cmd;
  float f1, f2, f3;

  FILE *fin = fopen(filename, "r");
  while (fgets(line, 400, fin)) {
    lcount++;
    if (sscanf(line, "%c %f %f %f", &cmd, &f1, &f2, &f3) == 4) {
      switch(cmd) {
      case 'v':
	obj->nverts++;
	obj->verts = (XYZ *)realloc(obj->verts, obj->nverts * sizeof(XYZ));
	obj->verts[obj->nverts-1].y = f1;
	obj->verts[obj->nverts-1].z = f2;
	obj->verts[obj->nverts-1].x = f3;
	break;

      case 'f':
	obj->nfacets++;
	obj->facets = (int *)realloc(obj->facets, obj->nfacets * 3 * sizeof(int));
	obj->facets[(obj->nfacets-1)*3 + 0] = (int)(f1+0.5);
	obj->facets[(obj->nfacets-1)*3 + 1] = (int)(f2+0.5);
	obj->facets[(obj->nfacets-1)*3 + 2] = (int)(f3+0.5);
	break;

      default:
	fprintf(stderr, "Unknown obj file cmd '%c' at line %d, ignoring.\n", cmd, lcount);
	break;

      }
    } else {
      fprintf(stderr, "Could not parse obj file at line %d, ignoring.\n", lcount);
    }
  }
  
  fprintf(stderr, "Read %d lines (%d facets) from obj file.\n", lcount, obj->nfacets);

  calcObjMinMaxMean(obj);

  return obj;
}

// load obj struct from *ASCII* freesurfer binary file, format descr here:
// http://wideman-one.com/gw/brain/fs/surfacefileformats.htm
// binary format was "awkward" to read because it had 3-byte ints: what is
// their endianness?!?!?!?!
OBJ_STRUCT *loadObjFromFS(char *filename, char *label, float r, float g, float b, float a) {
  OBJ_STRUCT *obj = (OBJ_STRUCT *)malloc(1 * sizeof(OBJ_STRUCT));

  strncpy(obj->label, label, OBJ_STRUCT_LABEL_LEN);
  obj->col.r = r;
  obj->col.g = g;
  obj->col.b = b;
  obj->alpha = a;

  obj->nverts = 0;
  obj->verts = NULL;
  obj->nfacets = 0;
  obj->facets = NULL;

  obj->norms = NULL;

  char line[401];
  float f1, f2, f3;
  int d1, d2, d3, d4;

  FILE *fin = fopen(filename, "r");

  fgets(line, 400, fin); // read comment
  fprintf(stderr, "read comment: %s\n", line);
  fgets(line, 400, fin); // read nvert, nface
  sscanf(line, "%d%d", &obj->nverts, &obj->nfacets);
  fprintf(stderr, "nverts = %d, nfacets = %d\n", obj->nverts, obj->nfacets);
  obj->verts = (XYZ *)malloc(obj->nverts * sizeof(XYZ));
  obj->facets = (int *)malloc(obj->nfacets * 3 * sizeof(int));
 
    
  // read verts
  int i;
  for (i = 0; i < obj->nverts; i++) {
    if (!fgets(line, 400, fin)) {
      fprintf(stderr, "failure reading file at vertex %d\n", i);
      exit(-1);
    }
    if (sscanf(line, "%f%f%f%d", &f1, &f2, &f3, &d4) != 4) {
      fprintf(stderr, "failure parsing file at vertex %d\n", i);
      exit(-1);
    }
    if (d4 != 0) {
      fprintf(stderr, "patch != 0\n");
      exit(-1);
    }
    if (!(i % 100)) {
      fprintf(stderr, "x = %f\n", f1);
    }
    obj->verts[i].x = f1;
    obj->verts[i].y = f2;
    obj->verts[i].z = f3;
  }

  // read facets
  for (i = 0; i < obj->nfacets; i++) {
    if (!fgets(line, 400, fin)) {
      fprintf(stderr, "failure reading file at facet %d\n", i);
      exit(-1);
    }
    if (sscanf(line, "%d%d%d%d", &d1, &d2, &d3, &d4) != 4) {
      fprintf(stderr, "failure parsing file at facet %d\n", i);
      exit(-1);
    }
    if (d4 != 0) {
      fprintf(stderr, "patch != 0\n");
      exit(-1);
    }
    obj->facets[i*3 + 0] = d1+1; //(int)(f1+0.5);
    obj->facets[i*3 + 1] = d2+1; //(int)(f2+0.5);
    obj->facets[i*3 + 2] = d3+1; //(int)(f3+0.5);
  }
  
  fprintf(stderr, "Read %d vertices (%d facets) from obj file.\n", obj->nverts, obj->nfacets);

  calcObjMinMaxMean(obj);

  return obj;
}

OBJ_WGT_STRUCT *loadObjWgtFromFS(char *filename) {
  OBJ_WGT_STRUCT *wgtobj = (OBJ_WGT_STRUCT *)malloc(1 * sizeof(OBJ_WGT_STRUCT));

  char line[401];
  wgtobj->minW = 9e99;
  wgtobj->maxW = -9e99;

  FILE *fin = fopen(filename, "r");

  fgets(line, 400, fin); // read (and ignore) "latency" line
  fgets(line, 400, fin); // read nvert
  sscanf(line, "%d", &wgtobj->nverts);
  fprintf(stderr, "nverts = %d\n", wgtobj->nverts);
  wgtobj->vert_indices = (int *)malloc(wgtobj->nverts * sizeof(int));
  wgtobj->vert_weights = (float *)malloc(wgtobj->nverts * sizeof(float));
  
  int i;
  for (i = 0; i < wgtobj->nverts; i++) {
    fgets(line, 400, fin);
    if (sscanf(line, "%d%f", &wgtobj->vert_indices[i], &wgtobj->vert_weights[i]) != 2) {
      fprintf(stderr, "Failed reading FS wgts file...\n");
      return NULL;
    }
    if (wgtobj->vert_weights[i] < wgtobj->minW) {
      wgtobj->minW = wgtobj->vert_weights[i];
    }
    if (wgtobj->vert_weights[i] > wgtobj->maxW) {
      wgtobj->maxW = wgtobj->vert_weights[i];
    }
  }

  return wgtobj;
}

typedef struct {
  float n[3];
  float vx[9];
  //unsigned short uu;
} /* __attribute__((packed)) */ STL_FACET_STRUCT;

#define VEPSI 0.0001

OBJ_STRUCT *loadObjFromSTL(char *filename, char *label, float r, float g, float b, float a) {

  OBJ_STRUCT *obj = (OBJ_STRUCT *)malloc(1 * sizeof(OBJ_STRUCT));

  strncpy(obj->label, label, OBJ_STRUCT_LABEL_LEN);
  obj->col.r = r;
  obj->col.g = g;
  obj->col.b = b;
  obj->alpha = a;

  obj->nverts = 0;
  obj->verts = NULL;
  obj->nfacets = 0;
  obj->facets = NULL;

  obj->norms = NULL;

  char title[81];
  
  FILE *fin = fopen(filename, "rb");

  fread(title, sizeof(char), 80, fin);
  title[80] = '\0';

  unsigned int nf;
  fread(&nf, sizeof(nf), 1, fin);

  STL_FACET_STRUCT *stl_facet = (STL_FACET_STRUCT *)malloc(nf * sizeof(STL_FACET_STRUCT));
  int j;
  unsigned short uu;
  for (j = 0; j < nf; j++) {
    if (fread(&(stl_facet[j]), sizeof(STL_FACET_STRUCT), 1, fin) != 1) {
      fprintf(stderr, "failed reading facets\n");
      return NULL;
    }
    if (fread(&uu, sizeof(unsigned short), 1, fin) != 1) {
      fprintf(stderr, "failed reading unused short\n");
      return NULL;
    }
    if (uu) {
      //fprintf(stderr, "value of unused short is wrong!\n");
    }
  }
  
  unsigned char tmp;
  fread(&tmp, 1, 1, fin);
  if (!feof(fin)) {
    fprintf(stderr, "Not at end of file!\n");
  }
  
  int fidx, iv, sv;
  XYZ iP;
  obj->nfacets = nf;
  obj->facets = (int *)malloc(obj->nfacets * 3 * sizeof(int));

  for (fidx = 0; fidx < nf; fidx++) {
    // loop over input vertices
    for (iv = 0; iv < 3; iv++) {
      iP.x = stl_facet[fidx].vx[iv * 3 + 0];
      iP.y = stl_facet[fidx].vx[iv * 3 + 1];
      iP.z = stl_facet[fidx].vx[iv * 3 + 2];
#if (1)
      // find matching stored vertex
      sv = obj->nverts-1;
      while ((sv > -1) && (VectorLength(obj->verts[sv], iP) > VEPSI)) {
	sv--;
      }
      if (sv < 0) {
	sv = obj->nverts;
	obj->nverts++;
	obj->verts = (XYZ *)realloc(obj->verts, obj->nverts * sizeof(XYZ));
      }
      if (sv < 0 || sv >= obj->nverts) {
	fprintf(stderr, " * * * \n");
      }
      obj->verts[sv] = iP;
      
      obj->facets[fidx * 3 + iv] = sv+1;
#endif
    }
  }
  
  calcObjMinMaxMean(obj);

  return obj;
}

void calcObjMinMaxMean(OBJ_STRUCT *obj) {

  XYZ minP = {9e99,9e99,9e99};
  XYZ maxP = {-9e99,-9e99,-9e99};
  XYZ meanP = {0., 0., 0.};
  int i;

  for (i = 0; i < obj->nverts; i++) {

    if (obj->verts[i].x < minP.x) {
      minP.x = obj->verts[i].x;
    }
    if (obj->verts[i].x > maxP.x) {
      maxP.x = obj->verts[i].x;
    }
    
    if (obj->verts[i].y < minP.y) {
      minP.y = obj->verts[i].y;
    }
    if (obj->verts[i].y > maxP.y) {
      maxP.y = obj->verts[i].y;
    }
    
    if (obj->verts[i].z < minP.z) {
      minP.z = obj->verts[i].z;
    }
    if (obj->verts[i].z > maxP.z) {
      maxP.z = obj->verts[i].z;
    }

    meanP.x += obj->verts[i].x;
    meanP.y += obj->verts[i].y;
    meanP.z += obj->verts[i].z;

  }

  meanP.x /= (float)obj->nverts;
  meanP.y /= (float)obj->nverts;
  meanP.z /= (float)obj->nverts;

  obj->minP = minP;
  obj->maxP = maxP;
  obj->meanP = meanP;

  fprintf(stderr, "Coordinate range is (%f,%f,%f) - (%f,%f,%f) [avg = (%f,%f,%f)]\n", 
	  obj->minP.x, obj->minP.y, obj->minP.z,
	  obj->maxP.x, obj->maxP.y, obj->maxP.z,
	  obj->meanP.x, obj->meanP.y, obj->meanP.z);

}

void calcVertexNormals(OBJ_STRUCT *obj) {

  if (obj->norms) {
    free(obj->norms);
  }
  obj->norms = (XYZ *)malloc(obj->nverts * sizeof(XYZ));

  if (!obj->norms) {
    fprintf(stderr, "failed to allocate storage for normals!\n");
    exit(-1);
  }

  int i, j;
  int fcount;
  XYZ norm_acc, n;
  for (i = 0; i < obj->nverts; i++) {
    
    // find facets that use this vertex and accumulate the normals 
    fcount = 0;
    norm_acc.x = norm_acc.y = norm_acc.z = 0.0;

    for (j = 0; j < obj->nfacets; j++) {
      if ((obj->facets[j*3+0]-1 == i) ||
	  (obj->facets[j*3+1]-1 == i) ||
	  (obj->facets[j*3+2]-1 == i)) {
	fcount++;
	n = CalcNormal(obj->verts[obj->facets[j*3+0]-1],
		       obj->verts[obj->facets[j*3+1]-1],
		       obj->verts[obj->facets[j*3+2]-1]);
	// what weight should we add this with? ... vertex area?
	// use Heron's formula
	float a = VectorLength(obj->verts[obj->facets[j*3+0]-1],
			       obj->verts[obj->facets[j*3+1]-1]);
	float b = VectorLength(obj->verts[obj->facets[j*3+1]-1],
			       obj->verts[obj->facets[j*3+2]-1]);
	float c = VectorLength(obj->verts[obj->facets[j*3+2]-1],
			       obj->verts[obj->facets[j*3+0]-1]);
	float s = 0.5 * (a+b+c);
	float wgt = sqrt(s*(s-a)*(s-b)*(s-c));
	VectorMul(n, wgt);
	norm_acc = VectorAdd(norm_acc, n);
      }
    }
    //obj->norms[i] = VectorMul(norm_acc, -1);
    obj->norms[i] = norm_acc;
    if (fcount) {
      Normalise(&(obj->norms[i]));
    }

  }

}


void translateObj(OBJ_STRUCT *obj, XYZ vec) {
  int i;
  for (i = 0; i < obj->nverts; i++) {
    obj->verts[i] = VectorSub(vec, obj->verts[i]);
  }
  obj->minP = VectorSub(vec, obj->minP);
  obj->maxP = VectorSub(vec, obj->maxP);
}

void drawObj(OBJ_STRUCT *obj) { // , COLOUR c) {

  int i, j;
  XYZ P[3];
  
  pushVRMLname(obj->label);

  for (i = 0; i < obj->nfacets; i++) {
    for (j = 0; j < 3; j++) {
      P[j] = obj->verts[obj->facets[i*3+j]-1];
    }
    if (obj->alpha < 0.999) {    
      ns2vf3a(P, obj->col, 's', obj->alpha);
    } else {
      if (obj->norms) {
	// have vertex normals, use them
	//fprintf(stderr, "using normals...\n");
	XYZ N[3];
	//N[0] = CalcNormal(P[0], P[1], P[2]);
	//N[1] = N[2] = N[0];
	N[0] = obj->norms[obj->facets[i*3+0]-1];
	N[1] = obj->norms[obj->facets[i*3+1]-1];
	N[2] = obj->norms[obj->facets[i*3+2]-1];
	ns2vf3n(P, N, obj->col);
      } else {
	ns2vf3(P, obj->col);
      }
    }
  }

  pushVRMLname("ANON");
}

void drawObjWgts(OBJ_STRUCT *obj, OBJ_WGT_STRUCT *wgt, int wgted_vx_only, 
		 int solid_col_idx) {
  //void drawObjWgts(OBJ_STRUCT *obj, OBJ_WGT_STRUCT *wgt) {

  int i, j, k;
  XYZ P[3];
  
  pushVRMLname(obj->label);

  int c1, c2, cidx;
  s2qcir(&c1, &c2);
  float r, g, b;

  COLOUR col;
  int draw_me;
  for (i = 0; i < obj->nfacets; i++) {
    draw_me = 0;
    col.r = col.g = col.b = 0.0;
    for (j = 0; j < 3; j++) {
      P[j] = obj->verts[obj->facets[i*3+j]-1];

      // do we have a wgt for this vertex?
      k = 0;
      while (k < wgt->nverts && wgt->vert_indices[k] != obj->facets[i*3+j]-1) {
	k++;
      }
      if (k < wgt->nverts) {
	draw_me = 1;
	// map wgt [0,1] to colour in map
	float w = (wgt->vert_weights[k] - wgt->minW) / (wgt->maxW - wgt->minW);
	cidx = c1 + (int)(w * (c2-c1) + 0.5);
	if (cidx < c1) {
	  cidx = c1;
	}
	if (cidx > c2) {
	  cidx = c2;
	}
	s2qcr(cidx, &r, &g, &b);
	col.r += 2*r;
	col.g += 2*g;
	col.b += 2*b;
      } else {
	col.r += obj->col.r;
	col.g += obj->col.g;
	col.b += obj->col.b;
      }
    }
    col.r *= 0.3333;
    if (col.r > 1.) {
      col.r = 1.;
    }
    col.g *= 0.3333;
    if (col.g > 1.) {
      col.g = 1.;
    }
    col.b *= 0.3333;
    if (col.b > 1.) {
      col.b = 1.;
    }

    if (solid_col_idx > -1) {
      s2qcr(solid_col_idx, &r, &g, &b);
      col.r = r;
      col.g = g;
      col.b = b;
    }

    if (!wgted_vx_only || draw_me) {
      if (obj->alpha < 0.999) { 
	ns2vf3a(P, col, 's', obj->alpha);
      } else {	  
	ns2vf3(P, col);
      }
    }
  }

  pushVRMLname("ANON");
  
}
