/* s2base.h: basic definitions used throughout S2PLOT library
 *
 * ==================================================================
 * S2PLOT is freely available for non-commercial and educational use,
 * however, it is not public-domain software. The programming library
 * and documentation may not be redistributed or sub-licensed in any
 * form without permission from Swinburne University of
 * Technology. The programming library and sample codes are provided
 * without warranty.
 *
 * We would appreciate it if research outcomes using S2PLOT would
 * provide the following acknowledgement:
 *
 * "Three-dimensional visualisation was conducted with the S2PLOT
 * progamming library"
 *
 * and a reference to 
 *
 * D.G.Barnes, C.J.Fluke, P.D.Bourke & O.T.Parry, 2006, Publications
 * of the Astronomical Society of Australia, 23(2), 82-93.
 *
 * S2PLOT development was funded through the Researcher Developer
 * Grant Scheme at Swinburne University of Technology.
 * ==================================================================
 *
 * original version: David Barnes, January 2008
 *
 * $Id: s2base.h 5722 2012-07-03 00:31:25Z dbarnes $
 *
 */

#ifndef S2BASE_H
#define S2BASE_H

#ifndef XYZSTRUCT
typedef struct {
#if defined(S2TRIPLEFLOAT)
  float x,y,z;
#else
  double x,y,z;
#endif
} XYZ;
#define XYZSTRUCT
#endif

#ifndef COLOURSTRUCT
typedef struct {
#if defined(S2TRIPLEFLOAT)
  float r,g,b,a;
#else
  double r,g,b;
#endif
} COLOUR;
#define COLOURSTRUCT
#endif

typedef struct {
  XYZ vp;           	/* View position            */
  XYZ vd;           	/* View direction vector    */
  XYZ vu;           	/* View up direction        */
  XYZ pr;		/* Point to rotate about    */
  double focallength;   /* Focal Length along vd    */
  XYZ focus;            /* Focal point - Derived    */
  double aperture;  	/* Camera VERTICAL aperture */
  double eyesep;	/* Eye separation	    */
  double speed;         /* Speed in vd direction    */
  double fishrotate;    /* Rotate fisheye camera    */
} CAMERA;


#endif
